﻿using DocumentGenerations.App_Code.DAL;
using Syncfusion.DocIO;
using Syncfusion.DocIO.DLS;
using System;
using System.Collections.Generic;
using System.IO;
using System.Linq;
using System.Text;
using System.Threading.Tasks;
using Windows.Storage;
using Windows.Storage.Pickers;
using Windows.Storage.Streams;

namespace DocumentGenerations.App_Code
{
    class GenerateDocuments
    {

        //Font font = new Font("Calibri", 21);

        Font font = new Font("Arial", 21);

        public async Task<WordDocument> ReadDocumentTemplate(string documentName)
        {
            WordDocument wrd = new WordDocument();
            try
            {
                StorageFolder packageLocation = Windows.ApplicationModel.Package.Current.InstalledLocation;
                StorageFolder documentFolder = await packageLocation.GetFolderAsync("DocumentTemp");
                StorageFile document = await documentFolder.GetFileAsync(documentName);

                var randomAccessStream = await document.OpenReadAsync();
                Stream cms = randomAccessStream.AsStreamForRead();

                wrd.Open(cms, FormatType.Docx);
            }
            catch (Exception ex)
            { }

            return wrd;
        }

        #region Save document at Sharepoint and Local Machine functionality

        public async Task SaveFileByPicker(WordDocument doc)
        {
            try
            {
                /// Create document in readonly mode and random password protected to edit
                doc.Protect(Syncfusion.DocIO.ProtectionType.AllowOnlyReading, GetPassword());

                MemoryStream ms = new MemoryStream();
                ms.Position = 0;

                await doc.SaveAsync(ms, FormatType.Docx);

                FileSavePicker SavePicker = new FileSavePicker();
                SavePicker.SuggestedStartLocation = PickerLocationId.Desktop;

                SavePicker.FileTypeChoices.Add("word document", new List<string>() { ".docx" });

                //DocumentModel.Load(isDocx ? "Document.docx" : "Document.doc").Save("Document.pdf");

                SavePicker.SuggestedFileName = "sample";
                StorageFile stgFile = await SavePicker.PickSaveFileAsync();

                if (stgFile != null)
                    using (IRandomAccessStream zipStream = await stgFile.OpenAsync(FileAccessMode.ReadWrite))
                    {
                        //  await document.SaveAsync(ms, FormatType.Docx);
                        using (Stream outstream = zipStream.AsStreamForWrite())
                        {

                            byte[] buffer = ms.ToArray();
                            outstream.Write(buffer, 0, buffer.Length);
                            outstream.Flush();
                            outstream.Position = 0;
                        }
                    }
            }
            catch (Exception ex)
            { }
        }

        public async void SaveFileByPicker(MemoryStream ms)
        {
            try
            {

                FileSavePicker SavePicker = new FileSavePicker();
                SavePicker.SuggestedStartLocation = PickerLocationId.Desktop;

                SavePicker.FileTypeChoices.Add("word document", new List<string>() { ".docx" });

                SavePicker.SuggestedFileName = "sample";
                StorageFile stgFile = await SavePicker.PickSaveFileAsync();

                if (stgFile != null)
                    using (IRandomAccessStream zipStream = await stgFile.OpenAsync(FileAccessMode.ReadWrite))
                    {
                        //  await document.SaveAsync(ms, FormatType.Docx);
                        using (Stream outstream = zipStream.AsStreamForWrite())
                        {
                            byte[] buffer = ms.ToArray();
                            outstream.Write(buffer, 0, buffer.Length);
                            outstream.Flush();
                            outstream.Position = 0;
                        }
                    }
            }
            catch (Exception ex)
            { }
        }

        public async void SaveFileByPicker(MemoryStream ms, string fileExtension)
        {
            try
            {

                FileSavePicker SavePicker = new FileSavePicker();
                SavePicker.SuggestedStartLocation = PickerLocationId.Desktop;

                SavePicker.FileTypeChoices.Add("word document", new List<string>() { "." + fileExtension });

                SavePicker.SuggestedFileName = "ProjectXMLFile";
                StorageFile stgFile = await SavePicker.PickSaveFileAsync();

                if (stgFile != null)
                    using (IRandomAccessStream zipStream = await stgFile.OpenAsync(FileAccessMode.ReadWrite))
                    {
                        //  await document.SaveAsync(ms, FormatType.Docx);
                        using (Stream outstream = zipStream.AsStreamForWrite())
                        {
                            byte[] buffer = ms.ToArray();
                            outstream.Write(buffer, 0, buffer.Length);
                            outstream.Flush();
                            outstream.Position = 0;
                        }
                    }
            }
            catch (Exception ex)
            { }
        }


        public string GetPassword()
        {
            StringBuilder builder = new StringBuilder();
            builder.Append(RandomString(4, true));
            builder.Append(RandomNumber(1000, 9999));
            builder.Append(RandomString(2, false));

            return builder.ToString();
        }

        private string RandomString(int size, bool lowerCase)
        {
            StringBuilder builder = new StringBuilder();
            Random random = new Random();
            char ch;
            for (int i = 0; i < size; i++)
            {
                ch = Convert.ToChar(Convert.ToInt32(Math.Floor(26 * random.NextDouble() + 65)));
                builder.Append(ch);
            }
            if (lowerCase)
                return builder.ToString().ToLower();
            return builder.ToString();
        }

        private int RandomNumber(int min, int max)
        {
            Random random = new Random();
            return random.Next(min, max);
        }

        #endregion


        #region Replace common iformation at Word documents

        public WordDocument ReplaceAllergiesValues(WordDocument wrd, List<AllergiesDAL> objAllergies)
        {
            try
            {
                if (objAllergies.Count > 0)
                {
                    WTable table = CreateNewTable(wrd, (objAllergies.Count + 1), 2); // Create Insurance table
                    table.TableFormat.ClearFormatting();

                    int rowCount = 0;

                    List<AllergiesDAL> objAllergy = objAllergies;
                    //Font font = new Font("Calibri", 11);

                    foreach (WTableRow row in table.Rows)
                    {
                        if (rowCount > 0)
                        {
                            IWTextRange value1 = row.Cells[0].AddParagraph().AppendText(objAllergy[rowCount - 1].ALLERGYTYPE.Trim()) as WTextRange;
                            value1.CharacterFormat.Font = font;

                            IWTextRange value2 = row.Cells[1].AddParagraph().AppendText(objAllergy[rowCount - 1].SEVERITY) as WTextRange;
                            value2.CharacterFormat.Font = font;
                        }
                        else
                        {

                            IWTextRange heading1 = row.Cells[0].AddParagraph().AppendText("Allergy Types") as WTextRange;
                            heading1.CharacterFormat.Font = font;
                            heading1.CharacterFormat.Bold = true;

                            IWTextRange heading2 = row.Cells[1].AddParagraph().AppendText("Severity") as WTextRange;
                            heading2.CharacterFormat.Font = font;
                            heading2.CharacterFormat.Bold = true;

                        }

                        rowCount++;

                    }

                    wrd = FindPlaceHolderAndInsertTable(wrd, table, "%$PatientAllergiesRuntimeData$%"); // Insert Allergies created table in document.
                }
                else
                {
                    wrd.Replace("%$PatientAllergiesRuntimeData$%", "No record found", true, true);
                }
            }
            catch (Exception ex)
            {
            }

            return wrd;

        }

        public WordDocument ReplaceDurableMedicalEquipmentValues(WordDocument wrd, List<EquipmentDAL> objEquipment)
        {

            try
            {
                int rowCount = 0;

                //if (CreateCaseSingleton.Instance.objEquipmentDAL.Count > 0)
                if (objEquipment.Count > 0)
                {
                    WTable table = CreateNewTable(wrd, (objEquipment.Count + 1), 6); // Create Table

                    //Font font = new Font("Calibri", 11);

                    foreach (WTableRow row in table.Rows)
                    {
                        if (rowCount > 0)
                        {
                            
                            WTextRange value1 = row.Cells[0].AddParagraph().AppendText(objEquipment[rowCount - 1].EQUIPMENTNAME.Trim()) as WTextRange;
                            value1.CharacterFormat.Font = font;

                            WTextRange value2 = row.Cells[1].AddParagraph().AppendText(objEquipment[rowCount - 1].ORDEREDFROM.Trim()) as WTextRange;
                            value2.CharacterFormat.Font = font;

                            WTextRange value3 = row.Cells[2].AddParagraph().AppendText(Convert.ToString(objEquipment[rowCount - 1].DMEPRICE).Trim()) as WTextRange;
                            value3.CharacterFormat.Font = font;

                            WTextRange value4 = row.Cells[3].AddParagraph().AppendText(Convert.ToString(objEquipment[rowCount - 1].DMEQUANTITY).Trim()) as WTextRange;
                            value4.CharacterFormat.Font = font;

                            WTextRange value5 = row.Cells[4].AddParagraph().AppendText(objEquipment[rowCount - 1].ORDEREDFDATE.Trim()) as WTextRange;
                            value5.CharacterFormat.Font = font;

                            WTextRange value6 = row.Cells[5].AddParagraph().AppendText(objEquipment[rowCount - 1].SUPPLYDATE.Trim()) as WTextRange;
                            value6.CharacterFormat.Font = font;

                        }
                        else
                        {
                            WTextRange heading1 = row.Cells[0].AddParagraph().AppendText("Name") as WTextRange;
                            heading1.CharacterFormat.Font = font;
                            heading1.CharacterFormat.Bold = true;

                            WTextRange heading2 = row.Cells[1].AddParagraph().AppendText("Ordered From") as WTextRange;
                            heading2.CharacterFormat.Font = font;
                            heading2.CharacterFormat.Bold = true;

                            WTextRange heading3 = row.Cells[2].AddParagraph().AppendText("Unit Price") as WTextRange;
                            heading3.CharacterFormat.Font = font;
                            heading3.CharacterFormat.Bold = true;

                            WTextRange heading4 = row.Cells[3].AddParagraph().AppendText("Quantity") as WTextRange;
                            heading4.CharacterFormat.Font = font;
                            heading4.CharacterFormat.Bold = true;

                            WTextRange heading5 = row.Cells[4].AddParagraph().AppendText("Ordered Date") as WTextRange;
                            heading5.CharacterFormat.Font = font;
                            heading5.CharacterFormat.Bold = true;

                            WTextRange heading6 = row.Cells[5].AddParagraph().AppendText("Supply Date") as WTextRange;
                            heading6.CharacterFormat.Font = font;
                            heading6.CharacterFormat.Bold = true;

                        }

                        rowCount++;

                    }

                    wrd = FindPlaceHolderAndInsertTable(wrd, table, "$%DMEInformationDataTable%$"); // Insert Treatment table in document.
                }
                else
                {
                    wrd.Replace("$%DMEInformationDataTable%$", "No record inserted", true, true);
                }
            }
            catch (System.Exception ex)
            {
            }

            return wrd;

        }

        #endregion

        //public async Task<WordDocument> AddPatientSignatureInDocuments(WordDocument wrd)
        //{
        //    try
        //    {
        //        //string imageName = "PatientSignature.jpg";

        //        //StorageFolder packageLocation = Windows.ApplicationModel.Package.Current.InstalledLocation;
        //        //StorageFolder documentFolder = await packageLocation.GetFolderAsync("DocumentTemp");

        //        //// Read image file
        //        //StorageFile imageFile = await documentFolder.GetFileAsync(imageName);
        //        //var imageStream = await imageFile.OpenReadAsync();
        //        //Stream stream = imageStream.AsStreamForRead();

        //        Stream stream = PatientVisitGlobalMembers.patientSignature;

        //        if (stream != null)
        //        {
        //            //Adds new section to the document
        //            IWSection section = wrd.AddSection();


        //            IWParagraph blankParagraph = section.AddParagraph();

        //            //Adds new paragraph to the section
        //            IWParagraph firstParagraph = section.AddParagraph();

        //            IWTextRange firsttext = firstParagraph.AppendText("Patient Signatures:");
        //            IWTextRange Pspace = firstParagraph.AppendText("\t");
        //            firsttext.CharacterFormat.FontSize = 14;
        //            firsttext.CharacterFormat.Bold = true;


        //            //Adds image to  the paragraph
        //            IWPicture picture = firstParagraph.AppendPicture(stream);

        //            //Sets height and width for the image
        //            picture.Height = 70;
        //            picture.Width = 70;


        //            IWTextRange datetext = firstParagraph.AppendText("\t\t\tDate:\t" + DateTime.Now.ToString("MM/dd/yyyy"));
        //            datetext.CharacterFormat.FontSize = 14;
        //            datetext.CharacterFormat.Bold = true;


        //            //Adding 2nd Paragraph 
        //            IWParagraph secParagraph = section.AddParagraph();

        //            IWTextRange sectext = secParagraph.AppendText("Visited By:\t");
        //            sectext.CharacterFormat.FontSize = 14;
        //            sectext.CharacterFormat.Bold = true;

        //            IWTextRange sectext1 = secParagraph.AppendText(string.Format("{0} {1}", GlobalMembers.FIRST_NAME, GlobalMembers.LAST_NAME));
        //            sectext1.CharacterFormat.FontSize = 14;
        //            sectext1.CharacterFormat.FontName = "Informal Roman";


        //            try
        //            {

        //                if (PatientVisitGlobalMembers.CurrWebHRAttendance.Count > 1)
        //                {
        //                    List<AddWebHRAttendanceDAL> objAtt = PatientVisitGlobalMembers.CurrWebHRAttendance;

        //                    //Adds new paragraph to the section for Appointment Start Position
        //                    IWParagraph grolocationStartParagraph = section.AddParagraph();

        //                    IWTextRange appStartPosition = grolocationStartParagraph.AppendText("Appointment Start Position:\t\t");
        //                    appStartPosition.CharacterFormat.FontSize = 14;
        //                    appStartPosition.CharacterFormat.Bold = true;

        //                    IWParagraph blankParagraph1 = section.AddParagraph();

        //                    IWTextRange latituteText = grolocationStartParagraph.AppendText("Latitute:\t" + objAtt[0].Latitute);
        //                    latituteText.CharacterFormat.FontSize = 14;
        //                    latituteText.CharacterFormat.Bold = true;

        //                    IWTextRange longituteText = grolocationStartParagraph.AppendText("\t\tLongitute:\t" + objAtt[0].longitute);
        //                    longituteText.CharacterFormat.FontSize = 14;
        //                    longituteText.CharacterFormat.Bold = true;



        //                    IWParagraph blankParagraph2 = section.AddParagraph();

        //                    //Adds new paragraph to the section for Appointment End Position
        //                    IWParagraph grolocationEndParagraph = section.AddParagraph();

        //                    IWTextRange appEndPosition = grolocationEndParagraph.AppendText("Appointment End Position:\t\t");
        //                    appEndPosition.CharacterFormat.FontSize = 14;
        //                    appEndPosition.CharacterFormat.Bold = true;

        //                    IWParagraph blankParagraph3 = section.AddParagraph();

        //                    IWTextRange endLatituteText = grolocationEndParagraph.AppendText("Latitute:\t" + objAtt[0].Latitute);
        //                    endLatituteText.CharacterFormat.FontSize = 14;
        //                    endLatituteText.CharacterFormat.Bold = true;


        //                    IWTextRange EndLongituteText = grolocationEndParagraph.AppendText("\t\tLongitute:\t" + objAtt[0].longitute);
        //                    EndLongituteText.CharacterFormat.FontSize = 14;
        //                    EndLongituteText.CharacterFormat.Bold = true;


        //                }

        //            }
        //            catch (Exception ex)
        //            {
        //            }



        //        }
        //    }
        //    catch (Exception ex)
        //    { }

        //    return wrd;

        //}

        /* -------------------------- Working with Word Document Tables at Runtime------------------------------------------------- */

        /// <summary>
        /// Place Created new table at specific place in document
        /// </summary>
        /// <param name="table"></param>
        /// <param name="placeHolderName"></param>
        public WordDocument FindPlaceHolderAndInsertTable(WordDocument wrd, WTable table, string placeHolderName)
        {
            //Add the table to the text body part.
            TextBodyPart bodyPart = new TextBodyPart(wrd);
            bodyPart.BodyItems.Add(table);

            //Replace the place holder text with the table.
            wrd.Replace(placeHolderName, bodyPart, true, true, true);

            return wrd;
        }

        /// <summary>
        /// Create new table.
        /// </summary>
        /// <param name="document"></param>
        /// <returns></returns>
        public WTable CreateNewTable(WordDocument wrd, int numOfRows, int numOfColumns)
        {
            //Create new table.
            WTable table = new WTable(wrd, true);
            //Set the number of rows and cells.
            table.ResetCells(numOfRows, numOfColumns);

            //InsertTextToTableCell(table, "NEWLY CREATED TABLE", font);

            return table;
        }

        /* --------------------------------------------------------------------------- */


    }
}
